# -*- coding: utf-8 -*-
# Copyright 2024 Matthew Fitzpatrick.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <https://www.gnu.org/licenses/gpl-3.0.html>.
r"""``czekitout`` is a Python library that contains a collection of functions
that facilitate validation, and type-conversions, with useful error messages
when exceptions are thrown.

"""



#####################################
## Load libraries/packages/modules ##
#####################################

# Import child modules and packages of current package.
import czekitout.name
import czekitout.isa
import czekitout.check
import czekitout.convert

# Get version of current package.
from czekitout.version import __version__



##################################
## Define classes and functions ##
##################################

# List of public objects in package.
__all__ = []



###########################
## Define error messages ##
###########################
